function auxdata = SetupOpenSimOptimalTracking(GPOPSResult,haveSpringConsts)
% If GPOPSResult = 1, it will load the result of a GPOPS simulation instead
% of IK and ID results. This simulation result comes from folder 3_1,
% dynamically consistant IK-ID.

close all
haveHandSpringConsts = false;
haveOptSynergies = true;

%% Load experimental data and osim analysis data

% Specify names of files containing necessary data
MkrDataFile = 'Trial04_cycle_12.trc'; % Must be TRC file
GRFDataFile = 'Trial04_cycle_12_GRF.mot'; % Must be mot or sto file
IKDataFile = 'Trial04_cycle_12_ik.mot'; % Must be mot or sto file
IDDataFile = 'Trial04_cycle_12_id.sto'; % Must be mot or sto file

% Load data from files
[MkrData, MkrNames, MkrRate, MkrTime] = readTRCFile(MkrDataFile);
[GRFData, GRFLabels, GRFTime] = readMotFile(GRFDataFile);
[IKData, IKLabels, IKTime] = readMotFile(IKDataFile);
[IDData, IDLabels, IDTime] = readMotFile(IDDataFile);

% Specify files containing stim trial data
MkrDataFile = 'FES01_cycle_33.trc';
GRFDataFile = 'FES01_cycle_33_GRF.mot'; % Must be mot or sto file
IKDataFile = 'FES01_cycle_33_ik.mot'; % Must be mot or sto file
IDDataFile = 'FES01_cycle_33_id.sto'; % Must be mot or sto file

% Load data from files
[MkrDataStim, MkrNamesStim, MkrRateStim, MkrTimeStim] = readTRCFile(MkrDataFile);
[GRFDataStim, GRFLabelsStim, GRFTimeStim] = readMotFile(GRFDataFile);
[IKDataStim, IKLabelsStim, IKTimeStim] = readMotFile(IKDataFile);
[IDDataStim, IDLabelsStim, IDTimeStim] = readMotFile(IDDataFile);

%% Format data

% Remove extra time frames from the data
MkrData = MkrData(21:121,:);
MkrTime = MkrTime(21:121,:);
GRFData = GRFData(21:121,:);
GRFTime = GRFTime(21:121,:);
IKData = IKData(21:121,:);
IKTime = IKTime(21:121,:);
IDData = IDData(21:121,:);
IDTime = IDTime(21:121,:);

MkrDataStim = MkrDataStim(21:121,:);
MkrTimeStim = MkrTimeStim(21:121,:);
GRFDataStim = GRFDataStim(21:121,:);
GRFTimeStim = GRFTimeStim(21:121,:);
IKDataStim = IKDataStim(21:121,:);
IKTimeStim = IKTimeStim(21:121,:);
IDDataStim = IDDataStim(21:121,:);
IDTimeStim = IDTimeStim(21:121,:);

% Convert IKData to radians
IKData(:,[1:3 7:end]) = IKData(:,[1:3 7:end])*pi/180;
IKDataStim(:,[1:3 7:end]) = IKDataStim(:,[1:3 7:end])*pi/180;

torques = IDData(:,[7:10,12:18,20:29,33:36]);
torqueNames = IDLabels([7:10,12:18,20:29,33:36]);
q = IKData(:,[1:10,12:18,20:29,33:36]);
qLabels = IKLabels([1:10,12:18,20:29,33:36]);
torquesStim = IDDataStim(:,[7:10,12:18,20:29,33:36]);
qStim = IKDataStim(:,[1:10,12:18,20:29,33:36]);


if GPOPSResult
    load GPOPSResults_Trial04_Cycle12.mat
    load handLoads_Trial04_Cycle12.mat
    
    auxdata.handLoadsR = handLoadsR;
    auxdata.handLoadsL = handLoadsL;
    auxdata.spHandLoadsR = spHandLoadsR;
    auxdata.spHandLoadsL = spHandLoadsL;
end

MkrNamesOSIM = {'R.Shoulder' 'L.Shoulder' 'Neck' 'Chest' 'R.Elbow' 'R.Wrist' 'L.Elbow' 'L.Wrist' 'R.ASIS' 'L.ASIS'...
    'Sacral' 'R.Thigh.Superior' 'R.Thigh.Inferior' 'R.Thigh.Lateral' 'R.Shank.Superior' 'R.Shank.Inferior' 'R.Shank.Lateral'...
    'R.Heel' 'R.Midfoot.Superior' 'R.Midfoot.Lateral' 'R.Toe' 'L.Thigh.Superior' 'L.Thigh.Inferior' 'L.Thigh.Lateral'...
    'L.Shank.Superior' 'L.Shank.Lateral' 'L.Shank.Inferior' 'L.Heel' 'L.Midfoot.Superior' 'L.Midfoot.Lateral' 'L.Toe'...
    'L.PSIS' 'R.PSIS'};

% Determine MarkerOrder
MkrDataOrder = zeros(1,numel(MkrNamesOSIM));
for i = 1:numel(MkrNamesOSIM)
    MkrDataOrder(i) = find(ismember(MkrNames, MkrNamesOSIM{i}));
end
% Create new reordered marker matrix
MkrDataR = zeros(101,length(MkrDataOrder)*3);
MkrDataRStim = zeros(101,length(MkrDataOrder)*3);
for i = 1:length(MkrDataOrder)
    j = MkrDataOrder(i);
    MkrDataR(:,(i-1)*3+1:3*i) = MkrData(:,(j-1)*3+1:3*j);
    MkrDataRStim(:,(i-1)*3+1:3*i) = MkrDataStim(:,(j-1)*3+1:3*j);
end

MkrNames = MkrNames(MkrDataOrder);
MkrData = MkrDataR;
MkrDataStim = MkrDataRStim;

% Create left and right GRF matrices
GRFDataL = GRFData(:,1:6);
ECL = GRFData(1,13:15);
GRFDataR = GRFData(:,7:12);
ECR = GRFData(1,16:18);

GRFDataL(abs(GRFDataL(:,2))<10,:)=0;
GRFDataR(abs(GRFDataR(:,2))<10,:)=0;

GRFDataLStim = GRFDataStim(:,1:6);
ECLStim = GRFDataStim(1,13:15);
GRFDataRStim = GRFDataStim(:,7:12);
ECRStim = GRFDataStim(1,16:18);

GRFDataLStim(abs(GRFDataL(:,2))<10,:)=0;
GRFDataRStim(abs(GRFDataR(:,2))<10,:)=0;

time = MkrTime-MkrTime(1);
timeStim = MkrTimeStim-MkrTimeStim(1);

% Get Coordinate Derivatives
if GPOPSResult
    qppp = zeros(size(q));
    for i = 1:size(q,2)
        [~, qppp(:,i)] = SplineSmooth(time,qpp(:,i),0.99999);
    end
else
    qp = zeros(size(q));
    qpp = zeros(size(q));
    qppp = zeros(size(q));
    for i = 1:size(q,2)
        [~, qp(:,i)] = SplineSmooth(time,q(:,i),0.99999); 
        [~, qpp(:,i)] = SplineSmooth(time,qp(:,i),0.99999);
        [~, qppp(:,i)] = SplineSmooth(time,qpp(:,i),0.99999);
    end
end


for i = 1:size(MkrData,2)
    [~, MkrDatap(:,i)] = SplineSmooth(time,MkrData(:,i),1);
end

for i = 1:size(qStim,2)
   [~, qpStim(:,i)] = SplineSmooth(time,qStim(:,i),0.99999); 
   [~, qppStim(:,i)] = SplineSmooth(time,qpStim(:,i),0.99999);
   [~, qpppStim(:,i)] = SplineSmooth(time,qppStim(:,i),0.99999);
end

for i = 1:size(MkrDataStim,2)
    [~, MkrDatapStim(:,i)] = SplineSmooth(time,MkrDataStim(:,i),1);
end

%% Define moment calculation point (midfoot marker projected onto the ground)

% Determine heel and toe marker coordinates
FootMkrNames = {'R.Midfoot.Superior' 'L.Midfoot.Superior'};

FootMkrs = zeros(1,numel(FootMkrNames));
for i = 1:numel(FootMkrNames)
    FootMkrs(i) = find(ismember(MkrNames, FootMkrNames{i}));
end

% Define points to calculate joint moments about
MCR = MkrDataR(:,3*(FootMkrs(1)-1)+1:3*FootMkrs(1));%(MkrDataR(:,3*(FootMkrs(1)-1)+1:3*FootMkrs(1))+MkrDataR(:,3*(FootMkrs(2)-1)+1:3*FootMkrs(2)))/2;
MCL = MkrDataR(:,3*(FootMkrs(2)-1)+1:3*FootMkrs(2));%(MkrDataR(:,3*(FootMkrs(3)-1)+1:3*FootMkrs(3))+MkrDataR(:,3*(FootMkrs(4)-1)+1:3*FootMkrs(4)))/2;

MCRStim = MkrDataRStim(:,3*(FootMkrs(1)-1)+1:3*FootMkrs(1));%(MkrDataR(:,3*(FootMkrs(1)-1)+1:3*FootMkrs(1))+MkrDataR(:,3*(FootMkrs(2)-1)+1:3*FootMkrs(2)))/2;
MCLStim = MkrDataRStim(:,3*(FootMkrs(2)-1)+1:3*FootMkrs(2));%(MkrDataR(:,3*(FootMkrs(3)-1)+1:3*FootMkrs(3))+MkrDataR(:,3*(FootMkrs(4)-1)+1:3*FootMkrs(4)))/2;

MCR(:,2) = 0;
MCL(:,2) = 0;

MCRStim(:,2) = 0;
MCLStim(:,2) = 0;

% Transfer GRFs to moment centers
GRFDataR(:,4:6) = cross(ones(101,1)*ECR-MCR,GRFDataR(:,1:3),2)+GRFDataR(:,4:6);
GRFDataL(:,4:6) = cross(ones(101,1)*ECL-MCL,GRFDataL(:,1:3),2)+GRFDataL(:,4:6);

GRFDataRStim(:,4:6) = cross(ones(101,1)*ECRStim-MCRStim,GRFDataRStim(:,1:3),2)+GRFDataRStim(:,4:6);
GRFDataLStim(:,4:6) = cross(ones(101,1)*ECLStim-MCLStim,GRFDataLStim(:,1:3),2)+GRFDataLStim(:,4:6);

%% Load spring locations

% Define body index where springs are located
%import OpenSim Matlab API functions
import org.opensim.modeling.*
import java.lang.*

inModel = 'Subject1_v4pt3_GPOPS.osim';

%setup OpenSim model
model = Model(inModel);
for iii = 1:41
    body = model.getBodySet.get(iii);
    if strcmp(body,'calcn_l')
        heelBodyL = iii;
    end
    if strcmp(body,'calcn_r')
        heelBodyR = iii;
    end
    if strcmp(body,'toes_l')
        toesBodyL = iii;
    end
    if strcmp(body,'toes_r')
        toesBodyR = iii;
    end
    if strcmp(body,'calcn_l_markerPlate')
        SpringBodyHL = iii;
    end
    if strcmp(body,'toes_l_markerPlate')
        SpringBodyTL = iii;
    end
    if strcmp(body,'calcn_r_markerPlate')
        SpringBodyHR = iii;
    end
    if strcmp(body,'toes_r_markerPlate')
        SpringBodyTR = iii;
    end
    if strcmp(body,'hand_r')
        handBodyR = iii;
    end
    if strcmp(body,'hand_l')
        handBodyL = iii;
    end
    if strcmp(body,'radius_r')
        radiusBodyR = iii;
    end
    if strcmp(body,'radius_l')
        radiusBodyL = iii;
    end
end
% SpringBodyHR = 28;
% SpringBodyTR = 39;
% SpringBodyHL = 27;
% SpringBodyTL = 38;


load spring_pos_for_polynomial_r.mat

SpringsPoly_r = [SpringsH; SpringsT];

SpringsPoly_r(:,1) = SpringsPoly_r(:,1)-min(SpringsPoly_r(:,1))-(max(SpringsPoly_r(:,1))-min(SpringsPoly_r(:,1)))/2;
SpringsPoly_r(:,3) = SpringsPoly_r(:,3)-min(SpringsPoly_r(:,3))-(max(SpringsPoly_r(:,3))-min(SpringsPoly_r(:,3)))/2;

load spring_pos_for_polynomial_l.mat

SpringsPoly_l = [SpringsH; SpringsT];

SpringsPoly_l(:,1) = SpringsPoly_l(:,1)-min(SpringsPoly_l(:,1))-(max(SpringsPoly_l(:,1))-min(SpringsPoly_l(:,1)))/2;
SpringsPoly_l(:,3) = SpringsPoly_l(:,3)-min(SpringsPoly_l(:,3))-(max(SpringsPoly_l(:,3))-min(SpringsPoly_l(:,3)))/2;

% SpringsPoly_l(:,1) = SpringsPoly_r(:,1);
% SpringsPoly_l(:,3) = -SpringsPoly_r(:,3);

load SpringLocsR.mat

SpringsHR = SpringsH;
SpringsTR = SpringsT;

nSprings = length(SpringsHR)+length(SpringsTR);


load SpringLocsL.mat

SpringsHL = SpringsH;
SpringsTL = SpringsT;

numSpringsBody = [length(SpringsHR) length(SpringsTR)];

nSprings = length(SpringsHR)+length(SpringsTR)+nSprings;


numSpringsBody = [numSpringsBody length(SpringsHL) length(SpringsTL)];
numSprings = sum(numSpringsBody);

auxdata.latchvel = 5e-2;

%% Load spring constants

auxdata.springMat = [zeros(4,3); 0.0865527 0.03508 -0.0160007;...
    0.0805573 0.0391741 0.00583226; SpringsHR; SpringsTR; SpringsHL; SpringsTL];
% auxdata.springBodyMat = [9;10;17;18;29;28;...
%     SpringBodyHR*ones(length(SpringsHR),1);...
%     SpringBodyTR*ones(length(SpringsTR),1);...
%     SpringBodyHL*ones(length(SpringsHL),1);...
%     SpringBodyTL*ones(length(SpringsTL),1)];
auxdata.springBodyMat = [heelBodyR;toesBodyR;heelBodyL;toesBodyL;28;27;...
    SpringBodyHR*ones(length(SpringsHR),1);...
    SpringBodyTR*ones(length(SpringsTR),1);...
    SpringBodyHL*ones(length(SpringsHL),1);...
    SpringBodyTL*ones(length(SpringsTL),1)];
auxdata.cVal = 1e-2*ones(1,numSprings);
auxdata.mu = 0.5;

auxdata.handBodyMat = [handBodyR;handBodyL];
auxdata.radiusBodyMat = [radiusBodyR;radiusBodyL];

if haveSpringConsts
    load SpringConstants.mat
end


%% Load EtM parameters that define activation to moment characteristics of muscles

load Subject1_v4pt1_EtM_params_trial04Cycle12.mat
% load AlteredExcitations.mat
whichLoadsEMG = [1 2 4 5 6 8 9 11 12 13];
whichLoadsActuators = 1:25;
whichLoadsActuators(:,whichLoadsEMG) = [];
% IDData(:,whichLoadsEMG) = [];
nEmg = size(emg,2);
nMuscTotal = size(a,2)/2;


% % switch order of EMG and activation data to be consistent with OpenSim
% % model
% a = [a(:,36:70) a(:,1:35)];
% emg = [emg(:,36:70) emg(:,1:35)];
% 
% % Switch order of EMGs since left leg is listed first in file and right leg
% % is calculated first in optimization.
% excitations = [excitations(1:101,36:70) excitations(1:101,1:35)];
% emgScaleOpt = [emgScaleOpt(:,36:70) emgScaleOpt(:,1:35)];
% tDelay = [tDelay(2) tDelay(1)];

% Reference indicating which muscles cross which joints
muscRef = zeros(1,nMuscTotal);
muscRef([1:16,32:33]) = 1; % Hip FE and AA
muscRef([17:19 21]) = 2; % Hip FE and AA and Knee FE
muscRef([20 22:24]) = 3; % Knee FE
muscRef([25:26]) = 4; % Knee FE, Ankle and Subtalar
muscRef([27:31,34:36]) = 5; % Ankle and Subtalar

muscRef = [muscRef muscRef+5];

muscleNames = {'addbrev_r','addlong_r','addmagDist_r','addmagIsch_r','addmagMid_r','addmagProx_r',...
    'glmax1_r','glmax2_r','glmax3_r','glmed1_r','glmed2_r','glmed3_r','glmin1_r','glmin2_r',...
    'glmin3_r','tfl_r','semimem_r','semiten_r','bflh_r','bfsh_r','recfem_r','vasmed_r','vaslat_r',...
    'vasint_r','gaslat_r','gasmed_r','tibant_r','perbrev_r','perlong_r','pertert_r','soleus_r',...
    'iliacus_r','psoas_r','tibpost_r','edl_r','fdl_r','addbrev_l','addlong_l','addmagDist_l'...
    ,'addmagIsch_l','addmagMid_l','addmagProx_l','glmax1_l','glmax2_l','glmax3_l','glmed1_l',...
    'glmed2_l','glmed3_l','glmin1_l','glmin2_l','glmin3_l','tfl_l','semimem_l','semiten_l','bflh_l',...
    'bfsh_l','recfem_l','vasmed_l','vaslat_l','vasint_l','gaslat_l','gasmed_l','tibant_l',...
    'perbrev_l','perlong_l','pertert_l','soleus_l','iliacus_l','psoas_l','tibpost_l','edl_l','fdl_l'};


%% Load Synergy Vectors

if haveOptSynergies
    load synergiesOpt.mat
    %synergyTime = synergyTime;
    synergyCommandsExp_r = neuralCommands_r;
    synergyCommandsExp_l = neuralCommands_l;
else
    % The weighting factors and commands given reproduce activations which have
    % been normalized by their standard deviation. In order to get actual
    % activations we need to multiply the synergy weights by their musle
    % activations standard deviation. This only applies to the muscles for
    % which we had emg.
    % NOTE (Nathan): Really should have saved the last used act stds for use
    % now, but I think this should work fine
    actStd = std(a,1);
    for iii = 1:31
        synergy.weights_r(:,iii) = synergy.weights_r(:,iii)*actStd(iii);
        synergy.weights_l(:,iii) = synergy.weights_l(:,iii)*actStd(iii+36);
    end

    synergyWeights_r = synergy.weights_r;
    synergyWeights_l = synergy.weights_l;
    synergyCommandsExp_r = synergy.commandsExp_r;
    synergyCommandsExp_l = synergy.commandsExp_l;
    
    synergyTime = time;
end



%% Load Stimulation parameters
load FES01_processed.mat
stimTrialTime = processedData.time{33}; % NOTE (Nathan): 33 is because we are using cycle 33 of trial FES01

% Stim on and off times do not necessarily match the index of the gait
% cycle, so we need to find the correct index
stimSwitchIndex = 1;
while processedData.stimOnTimes(stimSwitchIndex) < stimTrialTime(21)
    stimSwitchIndex = stimSwitchIndex + 1;
end
stimTrial_stimOnTime = processedData.stimOnTimes(stimSwitchIndex);
stimTrial_stimOffTime = processedData.stimOffTimes(stimSwitchIndex);

gasStim_tOn = stimTrial_stimOnTime - stimTrialTime(21);
gasStim_tOff = gasStim_tOn + 0.06;

taStim_tOff = stimTrial_stimOffTime - stimTrialTime(21);
taStim_tOn = taStim_tOff - 0.6;

gasStim_cyclePercentOn = gasStim_tOn/timeStim(end);
gasStim_cyclePercentOff = gasStim_tOff/timeStim(end);
taStim_cyclePercentOn = taStim_tOn/timeStim(end);
taStim_cyclePercentOff = taStim_tOff/timeStim(end);

load stimActFuncParams.mat
auxdata.gasStim_optParams = optParams_gas;
auxdata.taStim_optParams = optParams_ta;

c1 = optParams_gas(1);
c2 = optParams_gas(2);
c3 = optParams_gas(3);
c4 = optParams_gas(4);
tOnOffset = optParams_gas(5);
tOnOffset2 = optParams_gas(6);
tOffOffset = optParams_gas(7);
tOffOffset2 = optParams_gas(8);
gasStim_fesAct = (tanh(c1*(timeStim-gasStim_tOn-tOnOffset)) - tanh(c2*(timeStim-gasStim_tOff-tOffOffset)) + ...
    c3./(exp(2*c1*(timeStim-gasStim_tOn-tOnOffset2))+exp(2*c1*(-timeStim+gasStim_tOn+tOnOffset2))) + ...
    c4./(exp(2*c2*(timeStim-gasStim_tOff-tOffOffset2))+exp(2*c2*(-timeStim+gasStim_tOff+tOffOffset2))))/2;

c1 = optParams_ta(1);
c2 = optParams_ta(2);
c3 = optParams_ta(3);
c4 = optParams_ta(4);
tOnOffset = optParams_ta(5);
tOnOffset2 = optParams_ta(6);
tOffOffset = optParams_ta(7);
tOffOffset2 = optParams_ta(8);
taStim_fesAct = (tanh(c1*(timeStim-taStim_tOn-tOnOffset)) - tanh(c2*(timeStim-taStim_tOff-tOffOffset)) + ...
    c3./(exp(2*c1*(timeStim-taStim_tOn-tOnOffset2))+exp(2*c1*(-timeStim+taStim_tOn+tOnOffset2))) + ...
    c4./(exp(2*c2*(timeStim-taStim_tOff-tOffOffset2))+exp(2*c2*(-timeStim+taStim_tOff+tOffOffset2))))/2 + ...
    (1-tanh(c2*(timeStim-taStim_tOff-tOffOffset+timeStim(end))) + ...
    c4./(exp(2*c2*(timeStim-taStim_tOff-tOffOffset2+timeStim(end)))+exp(2*c2*(-timeStim+taStim_tOff+tOffOffset2-timeStim(end)))))/2;


load stimActFuncParams_all.mat
auxdata.stimOptParams = optParams;

load stimActFuncParams_all_simple.mat
auxdata.stimOptParams_simple = optParams;


%% Scale variables using canonical scaling

tscale = 1;
tscale2 = tscale^2;
tscale3 = tscale^3;
Lscale = 1; %Note, cannot scale length becuase it requires changing osim file
Mscale = 1; %Note, cannot scale mass becuase it requires changing osim file

% Scale Time
time = time*tscale;

% Scale velocities
qp = qp/tscale;
% Scale accelerations
qpp = qpp/tscale2;
% Scale forces and torques
GRFDataR = GRFDataR/tscale2;
GRFDataL = GRFDataL/tscale2;
IDData = IDData/tscale2;
% Scale spring constants
% Kvals = Kvals/tscale2;
auxdata.latchvel = auxdata.latchvel/tscale;

%% Create curvature minimizing data splines for use in optimal control problem

[spGRFR, GRFRNewData] = spaps(time,GRFDataR',1/tscale2);
[spGRFL, GRFLNewData] = spaps(time,GRFDataL',1/tscale2);
[spMkrData, MkrNewData] = spaps(time,MkrData',.00001);
[spIK, IKNewData] = spaps(time,IKData',.01*pi/180);
[spID, IDNewData] = spaps(time,IDData',0.1/tscale2);
[spTorques, TorquesNew] = spaps(time,torques',0.00001);
[spq, qNewData] = spaps(time,q',0.0000001);
[spqp, qpNewData] = spaps(time,qp',0.001);
[spqpp, qppNewData] = spaps(time,qpp',1/tscale2);
[spqppp, qpppNewData] = spaps(time,qppp',1/(tscale^3));
[spEMG, EMGNewData] = spaps(time,excitations',.00001);
[spActs, ActsNewData] = spaps(time,a',.00001);
[spMCR,MCRnew] = spaps(time,MCR',0.0000001);
[spMCL,MCLnew] = spaps(time,MCL',0.0000001);
[spSynergyCommands_l,synergyCommands_l_new] = spaps(synergyTime,synergyCommandsExp_l',0.00001);
[spSynergyCommands_r,synergyCommands_r_new] = spaps(synergyTime,synergyCommandsExp_r',0.00001);
synergyCommands_l_new = fnval(spSynergyCommands_l,time)';
synergyCommands_r_new = fnval(spSynergyCommands_r,time)';

[spGRFRStim, GRFRNewDataStim] = spaps(timeStim,GRFDataRStim',30/tscale2);
[spGRFLStim, GRFLNewDataStim] = spaps(timeStim,GRFDataLStim',30/tscale2);
[spMkrDataStim, MkrNewDataStim] = spaps(timeStim,MkrDataStim',.00001);
[spIKStim, IKNewDataStim] = spaps(timeStim,IKDataStim',.01*pi/180);
[spIDStim, IDNewDataStim] = spaps(timeStim,IDDataStim',0.1/tscale2);
[spTorquesStim, TorquesNewStim] = spaps(timeStim,torquesStim',0.00001);
[spqStim, qNewDataStim] = spaps(timeStim,qStim',0.0000001);
[spqpStim, qpNewDataStim] = spaps(timeStim,qpStim',0.001);
[spqppStim, qppNewDataStim] = spaps(timeStim,qppStim',1/tscale2);
[spqpppStim, qpppNewDataStim] = spaps(timeStim,qpppStim',1/(tscale^3));
[spMCRStim,MCRnewStim] = spaps(timeStim,MCRStim',0.0000001);
[spMCLStim,MCLnewStim] = spaps(timeStim,MCLStim',0.0000001);

[spGasStim_fesAct,gasStim_fesActNewData] = spaps(timeStim,gasStim_fesAct',0.0000001);
[spTaStim_fesAct,taStim_fesActNewData] = spaps(timeStim,taStim_fesAct',0.0000001);


% figure;
% for iii = 1:5
%     subplot(2,3,iii);
%     plot(time/time(end),synergyCommands_r_new(:,iii),'r');
%     hold on
%     plot(synergyTime/synergyTime(end),synergyCommandsExp_r(:,iii),'b');    
% end
% figure;
% for iii = 1:5
%     subplot(2,3,iii);
%     plot(time/time(end),synergyCommands_l_new(:,iii),'r');
%     hold on
%     plot(synergyTime/synergyTime(end),synergyCommandsExp_l(:,iii),'b');    
% end
% keyboard;
% plot(timeStim,GRFDataRStim(:,1:3),'b',timeStim,GRFRNewDataStim(1:3,:)','r')
% figure;
% plot(timeStim,GRFDataLStim(:,1:3),'b',timeStim,GRFLNewDataStim(1:3,:)','r')
% figure;
% plot(timeStim,GRFDataRStim(:,4:6),'b',timeStim,GRFRNewDataStim(4:6,:)','r')
% figure;
% plot(timeStim,GRFDataLStim(:,4:6),'b',timeStim,GRFLNewDataStim(4:6,:)','r')

%% Set output auxiliary data matrix

auxdata.time = time; %time vector
auxdata.spGRFR = spGRFR; % right ground reactions spline
auxdata.spGRFL = spGRFL; % left ground reactions spline
auxdata.grfr = GRFDataR;
auxdata.grfl = GRFDataL;
auxdata.spMkrData = spMkrData; % marker data spline
auxdata.spIK = spIK; % IK data spline
auxdata.spID = spID; % ID data spline
auxdata.spTorques = spTorques;
auxdata.spq = spq; % joint angles spline
auxdata.spqp = spqp; % joint velocities spline
auxdata.spqpp = spqpp; % joint accels spline
auxdata.spqppp = spqppp; % Joint jerk spline
auxdata.springsPoly_r = SpringsPoly_r;
auxdata.springsPoly_l = SpringsPoly_l;
auxdata.springsHR = SpringsHR; % spring locations right heel
auxdata.springsTR = SpringsTR; % spring locations right toes
auxdata.springsHL = SpringsHL; % spring locations left heel
auxdata.springsTL = SpringsTL; % spring locations left toes
auxdata.springBodyHR = SpringBodyHR; % index number of right heel body in opensim
auxdata.springBodyTR = SpringBodyTR; % index number of right toes body in opensim
auxdata.springBodyHL = SpringBodyHL; % index number of left heel body in opensim
auxdata.springBodyTL = SpringBodyTL; % index number of left toes body in opensim
auxdata.grfDataR = GRFDataR; % GRF data right
auxdata.grfDataL = GRFDataL; % GRF data left
auxdata.mkrData = MkrData; % marker data
auxdata.ikData = IKData; % IK data
auxdata.idData = IDData; % ID data
auxdata.torques = torques;
auxdata.torqueNames = torqueNames;
auxdata.q = q; % joint angles
auxdata.qp = qp; % joint velocities
auxdata.qpp = qpp; % joint accelerations
auxdata.qppp = qppp; % joint jerk
auxdata.qLabels = qLabels;
auxdata.mkrNames = MkrNames; % marker names
auxdata.numSprings = nSprings; % number of springs
auxdata.numSpringsBody = numSpringsBody; % number of springs on each foot body
auxdata.ecr = ECR; % right force plate electrical center
auxdata.ecl = ECL; % left force plate electrical center
auxdata.mcr = MCR; % right moment calculation point
auxdata.mcl = MCL; % left moment calculation point
auxdata.spMcr = spMCR; % right moment calculation point spline
auxdata.spMcl = spMCL; % left moment calculation point spline
auxdata.tScale = tscale; % time scale factor
if haveSpringConsts
    auxdata.kVal = kVal; % spring constants
    auxdata.yVal = yVal; % Spring height off of foot
end
if haveHandSpringConsts
    load HandSpringConstants.mat
    load HandSpringPosCorrections.mat
    auxdata.handSpring_k = handSpring_k;
    auxdata.handSpring_tk = handSpring_tk;
    auxdata.handPosCorrect_r = handPosCorrect_r;
    auxdata.handAngCorrect_r = handAngCorrect_r;
    auxdata.handPosCorrect_l = handPosCorrect_l;
    auxdata.handAngCorrect_l = handAngCorrect_l;
end
auxdata.muscleNames = muscleNames;
auxdata.lmo = lmo; % muscle optimal fiber length
auxdata.lts = lts; % muscle tendon slack length
auxdata.fMax = fMax; % muscle maximum force
auxdata.pennAngle = pennAngle; % muscle pennation angle
auxdata.etmCoefs = coefs; % coefficients defining muscle lengths and moment arms as functions of kinematics
auxdata.tAct = tAct; % muscle activation time constants
auxdata.tDelay = tDelay; % muscle time delay
auxdata.aNonlin = aNonlin; % muscle nonlinearity parameter
auxdata.emg = emg; % processed and normalized EMG data
auxdata.acts = a;
auxdata.excitations = excitations;
auxdata.spEmg = spEMG; % EMG data spline
auxdata.spActs = spActs;
auxdata.whichLoadsEmg = whichLoadsEMG; % define which joints are actuated by EMG
auxdata.whichLoadsActuators = whichLoadsActuators; % which joints are actuated by torques
auxdata.muscRef = muscRef; % Defines functions of muscles (which joints each muscle crosses)
auxdata.emgScaleOpt = emgScaleOpt;
auxdata.synergyCommands_l = synergyCommands_l_new;
auxdata.synergyCommands_r = synergyCommands_r_new;
auxdata.spSynergyCommands_l = spSynergyCommands_l;
auxdata.spSynergyCommands_r = spSynergyCommands_r;
auxdata.synergyWeights_l = synergyWeights_l;
auxdata.synergyWeights_r = synergyWeights_r;

auxdata.gasStimTOn = gasStim_cyclePercentOn*time(end);
auxdata.gasStimTOff = gasStim_cyclePercentOff*time(end);
auxdata.taStimTOn = taStim_cyclePercentOn*time(end);
auxdata.taStimTOff = taStim_cyclePercentOff*time(end);

auxdata.stim.time = timeStim; %time vector
auxdata.stim.spGRFR = spGRFRStim; % right ground reactions spline
auxdata.stim.spGRFL = spGRFLStim; % left ground reactions spline
auxdata.stim.grfr = GRFRNewDataStim;
auxdata.stim.grfl = GRFLNewDataStim;
auxdata.stim.spMkrData = spMkrDataStim; % marker data spline
auxdata.stim.spIK = spIKStim; % IK data spline
auxdata.stim.spID = spIDStim; % ID data spline
auxdata.stim.spTorques = spTorquesStim;
auxdata.stim.spq = spqStim; % joint angles spline
auxdata.stim.spqp = spqpStim; % joint velocities spline
auxdata.stim.spqpp = spqppStim; % joint accels spline
auxdata.stim.spqppp = spqpppStim; % Joint jerk spline
auxdata.stim.grfDataR = GRFDataRStim; % GRF data right
auxdata.stim.grfDataL = GRFDataLStim; % GRF data left
auxdata.stim.mkrData = MkrDataStim; % marker data
auxdata.stim.ikData = IKDataStim; % IK data
auxdata.stim.idData = IDDataStim; % ID data
auxdata.stim.torques = torquesStim;
auxdata.stim.q = qStim; % joint angles
auxdata.stim.qp = qpStim; % joint velocities
auxdata.stim.qpp = qppStim; % joint accelerations
auxdata.stim.qppp = qpppStim; % joint jerk
auxdata.stim.ecr = ECRStim; % right force plate electrical center
auxdata.stim.ecl = ECLStim; % left force plate electrical center
auxdata.stim.mcr = MCRStim; % right moment calculation point
auxdata.stim.mcl = MCLStim; % left moment calculation point
auxdata.stim.spMcr = spMCRStim; % right moment calculation point spline
auxdata.stim.spMcl = spMCLStim; % left moment calculation point spline

auxdata.stim.gasStimTOn = gasStim_tOn;
auxdata.stim.gasStimTOff = gasStim_tOff;
auxdata.stim.taStimTOn = taStim_tOn;
auxdata.stim.taStimTOff = taStim_tOff;

auxdata.gasStim_optParams = optParams_gas;
auxdata.taStim_optParams = optParams_ta;
auxdata.gasStim_fesAct = gasStim_fesAct;
auxdata.taStim_fesAct = taStim_fesAct;
auxdata.spGasStim_fesAct = spGasStim_fesAct;
auxdata.spTaStim_fesAct = spTaStim_fesAct;

end

function [Data, ColumnLabels, Time] = readMotFile(inFile)

%open file
fid = fopen(inFile);

%skip initial lines
line = 'asdf';
while ~strcmpi(line,'ENDHEADER')
    line = fgetl(fid);
    if length(line)>8
        line = line(1:9);
    end
end

% Find the labels on the columns
ColumnLabels = fgetl(fid);
ColumnLabels = textscan(ColumnLabels, '%t');
ColumnLabels = ColumnLabels{1}';

% Read numeric data
Data = fscanf(fid, '%f');

% Find length of data
nColumns = length(ColumnLabels);
nRows = length(Data)/nColumns;

% Reshape data into matrix
Data = reshape(Data, nColumns, nRows)';

% Extract time vector
Time = Data(:,1);
Data(:,1) = [];

% Remove time label from column labels
ColumnLabels(1) = [];

fclose(fid);
end

function [MarkerData, MarkerNames, SampleRate, Time] = readTRCFile(inFile)

% Read 5 header lines from input .trc file
fid = fopen(inFile);
header1 = fgetl(fid);
header2 = fgetl(fid);
header3 = fgetl(fid);
header4 = fgetl(fid);
header5 = fgetl(fid);

% Read marker data as one long column
data = fscanf(fid, '%f');
fclose(fid);

% Reshape marker data based on number of rows and
% columns of marker data specified in the input data
% file, with two extra columns for frame and time
info = sscanf(header3,'%f %f %d %d');
nrows = info(3,1);
ncols = info(4,1)*3;
data = reshape(data, ncols+2, nrows)';

SampleRate = info(1,1);

MarkerNames = strrep(header4,'Patient 4:','');
MarkerNames = textscan(MarkerNames,'%s');
MarkerNames = MarkerNames{1};
MarkerNames(1:2) = [];

Time = data(:,2);
MarkerData = data(:,3:end);
end