function findStimActFuncParams
close all

% Find constants that make the smooth stim activation function like the non
% smooth one for the specific TA and Gastroc muscle params

% Load data
load Subject1_v4pt1_EtM_params_trial04Cycle12.mat

% Extract needed parameters
% Indices are 26 (Gastroc) and 27 (TA)
tAct_gas = tAct(26);
tAct_ta = tAct(27);
tDeact_gas = 4*tAct_gas;
tDeact_ta = 4*tAct_ta;

tOn_gas = 0.47;
tOn_ta = 0.7;
tOff_gas = tOn_gas + 0.06;
tOff_ta = tOn_ta + 0.6;

% Create guess and bounds for the vector of unkowns
optParams_labels = {'c1','c2','c3','c4','tOnOffset','tOnOffset2','tOffOffset'};
unknowns_guess = [1,1,1,1,0,0,0,0];
unknowns_lb = [0.01,0.01,0.01,0.01,0,0,0,0];
unknowns_ub = [100,100,1.6,2,0.1,0.1,0.1,0.1];

options = optimoptions('fmincon','Algorithm','interior-point');

%% Define objective for activation shape (gastroc)
t_gas = linspace(0,1,101)';
act_gas = zeros(101,1);
act_gas(t_gas>=tOn_gas) = 1-exp(-(1/tAct_gas)*(t_gas(t_gas>=tOn_gas)-tOn_gas));
act_gas(t_gas>=tOff_gas) = exp(-(1/tDeact_gas)*(t_gas(t_gas>=tOff_gas)-tOff_gas));

% % Run lsqnonlin for the gastroc
% unknowns_guess = [0.06,25,100];
optParams_gas = fmincon(@(unknownsVec)stimParamSqCostFunc(unknownsVec,act_gas,t_gas,tOn_gas,tOff_gas),...
    unknowns_guess,[],[],[],[],unknowns_lb,unknowns_ub,...
    @(unknownsVec)nonlinearConstraints(unknownsVec,t_gas,tOn_gas,tOff_gas),options);
% optParams_gas = [0.1,50,100];

% Calculate optimized smooth activation function (gastroc)
% tSmooth = linspace(-1,1,101)';
% dt = optParams_gas(1);
% c1 = optParams_gas(2);
% c2 = optParams_gas(3);
% 
% actSmooth_gas = 1./((exp(c1*(tSmooth-dt))+1).*(exp(c2*(-tSmooth-dt))+1));
c1 = optParams_gas(1);
c2 = optParams_gas(2);
c3 = optParams_gas(3);
c4 = optParams_gas(4);
tOnOffset = optParams_gas(5);
tOnOffset2 = optParams_gas(6);
tOffOffset = optParams_gas(7);
tOffOffset2 = optParams_gas(8);
actSmooth_gas = (tanh(c1*(t_gas-tOn_gas-tOnOffset)) - tanh(c2*(t_gas-tOff_gas-tOffOffset)) + ...
    c3./(exp(2*c1*(t_gas-tOn_gas-tOnOffset2))+exp(2*c1*(-t_gas+tOn_gas+tOnOffset2))) + ...
    c4./(exp(2*c2*(t_gas-tOff_gas-tOffOffset2))+exp(2*c2*(-t_gas+tOff_gas+tOffOffset2))))/2;

% Plot smooth vs non-smooth activations (gastroc)
figure;
plot(t_gas,act_gas,'k',t_gas,actSmooth_gas,'b--');


%% Define objective for activation shape (TA)
t_ta = linspace(0,2,101)';
act_ta = zeros(101,1);
act_ta(t_ta>=tOn_ta) = 1-exp(-(1/tAct_ta)*(t_ta(t_ta>=tOn_ta)-tOn_ta));
act_ta(t_ta>=tOff_ta) = exp(-(1/tDeact_ta)*(t_ta(t_ta>=tOff_ta)-tOff_ta));

% % Run lsqnonlin for the TA
% unknowns_guess = [0.3,25,100];
optParams_ta = fmincon(@(unknownsVec)stimParamSqCostFunc(unknownsVec,act_ta,t_ta,tOn_ta,tOff_ta),...
    unknowns_guess,[],[],[],[],unknowns_lb,unknowns_ub,...
    @(unknownsVec)nonlinearConstraints(unknownsVec,t_ta,tOn_ta,tOff_ta),options);
% optParams_ta = [0.4,20,50];

% Calculate optimized smooth activation function (TA)
% tSmooth = linspace(-1,1,101)';
% dt = optParams_ta(1);
% c1 = optParams_ta(2);
% c2 = optParams_ta(3);
% 
% actSmooth_ta = 1./((exp(c1*(tSmooth-dt))+1).*(exp(c2*(-tSmooth-dt))+1));
c1 = optParams_ta(1);
c2 = optParams_ta(2);
c3 = optParams_ta(3);
c4 = optParams_ta(4);
tOnOffset = optParams_ta(5);
tOnOffset2 = optParams_ta(6);
tOffOffset = optParams_ta(7);
tOffOffset2 = optParams_ta(8);
actSmooth_ta = (tanh(c1*(t_ta-tOn_ta-tOnOffset)) - tanh(c2*(t_ta-tOff_ta-tOffOffset)) + ...
    c3./(exp(2*c1*(t_ta-tOn_ta-tOnOffset2))+exp(2*c1*(-t_ta+tOn_ta+tOnOffset2))) + ...
    c4./(exp(2*c2*(t_ta-tOff_ta-tOffOffset2))+exp(2*c2*(-t_ta+tOff_ta+tOffOffset2))))/2;

% Plot smooth vs non-smooth activations (TA)
figure;
plot(t_ta,act_ta,'k',t_ta,actSmooth_ta,'b--');


save stimActFuncParams.mat optParams_gas optParams_ta optParams_labels

end

%% Cost Functions and Constraints

function costVec = stimParamCostFunc(unknownsVec,act_objective,t,ton,toff)

% t = linspace(-1,1,101)';
% dt = unknownsVec(1);
% c1 = unknownsVec(2);
% c2 = unknownsVec(3);
% 
% act = 1./((exp(c1*(t-dt))+1).*(exp(c2*(-t-dt))+1));

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = 0.0;
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset))+exp(2*c2*(-t+toff+tOffOffset))))/2;

costVec = act - act_objective;

end

function costVec = stimParamSqCostFunc(unknownsVec,act_objective,t,ton,toff)

% t = linspace(-1,1,101)';
% dt = unknownsVec(1);
% c1 = unknownsVec(2);
% c2 = unknownsVec(3);
% 
% act = 1./((exp(c1*(t-dt))+1).*(exp(c2*(-t-dt))+1));

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = unknownsVec(7);
tOffOffset2 = unknownsVec(8);
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset2))+exp(2*c2*(-t+toff+tOffOffset2))))/2;

costVec = sum((act - act_objective).^2);

end


function [c,cEq] = nonlinearConstraints(unknownsVec,t,ton,toff)

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = unknownsVec(7);
tOffOffset2 = unknownsVec(8);
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset2))+exp(2*c2*(-t+toff+tOffOffset2))))/2;

c = [-act;act-1]; % c <= 0, and we want 0 <= act <= 1;
cEq = []; % No non-linear equality constraints
end


