function findStimActFuncParams_all
% close all

% Find constants that make the smooth stim activation function like the non
% smooth one for the specific TA and Gastroc muscle params

% Load data
load Subject1_v4pt1_EtM_params_trial04Cycle12.mat
tAct(tAct<0.0077) = 0.0077;
% tAct(tAct<0.008) = 0.008;
tDeact = 4*tAct;

% Create guess and bounds for the vector of unkowns
optParams_labels = {'c1','c2','c3','c4','tOnOffset','tOnOffset2','tOffOffset'};
unknowns_guess = [1,1,1,1,0,0,0,0];
unknowns_lb = [0.01,0.01,0.01,0.01,-0.1,-0.05,-0.1,-0.05];
% unknowns_ub = [100,100,1.6,2,0.1,0.1,0.1,0.1];
unknowns_ub = [20,20,1,1,0.1,0.05,0.1,0.05];

options = optimoptions('fmincon','Algorithm','sqp','MaxFunEvals',60000); %'active-set' 'interior-point' ,'MaxFunEvals',60000

t = linspace(0,1.5,101)';
tOn = [0.3,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,0.4,...
    0.4,0.4,0.4,0.3,0.4,0.4,0.4,0.4,0.4,0.4,...
    0.4,0.4,0.4,0.3,0.3,0.3,0.4,0.4,0.4,0.4,0.4];
tOff = tOn + 0.4;

optParams = cell(1,31);
figure;
for iii = 1:31
    act = zeros(101,1);
    act(t>=tOn(iii)) = 1-exp(-(1/tAct(iii))*(t(t>=tOn(iii))-tOn(iii)));
    act(t>=tOff(iii)) = exp(-(1/tDeact(iii))*(t(t>=tOff(iii))-tOff(iii)));
    
    optParams{iii} = fmincon(@(unknownsVec)stimParamSqCostFunc(unknownsVec,act,t,tOn(iii),tOff(iii)),...
    unknowns_guess,[],[],[],[],unknowns_lb,unknowns_ub,...
    @(unknownsVec)nonlinearConstraints(unknownsVec,t,tOn(iii),tOff(iii)),options);
    
    c1 = optParams{iii}(1);
    c2 = optParams{iii}(2);
    c3 = optParams{iii}(3);
    c4 = optParams{iii}(4);
    tOnOffset = optParams{iii}(5);
    tOnOffset2 = optParams{iii}(6);
    tOffOffset = optParams{iii}(7);
    tOffOffset2 = optParams{iii}(8);
    actSmooth = (tanh(c1*(t-tOn(iii)-tOnOffset)) - tanh(c2*(t-tOff(iii)-tOffOffset)) + ...
        c3./(exp(2*c1*(t-tOn(iii)-tOnOffset2))+exp(2*c1*(-t+tOn(iii)+tOnOffset2))) + ...
        c4./(exp(2*c2*(t-tOff(iii)-tOffOffset2))+exp(2*c2*(-t+tOff(iii)+tOffOffset2))))/2;

    % Plot smooth vs non-smooth activations (gastroc)
    subplot(6,6,iii);
    plot(t,act,'k',t,actSmooth,'b');
    ylim([-0.01, 1.01]);
    
    iii = iii
end


save stimActFuncParams_all.mat optParams optParams_labels


end

%% Cost Functions and Constraints

function costVec = stimParamCostFunc(unknownsVec,act_objective,t,ton,toff)

% t = linspace(-1,1,101)';
% dt = unknownsVec(1);
% c1 = unknownsVec(2);
% c2 = unknownsVec(3);
% 
% act = 1./((exp(c1*(t-dt))+1).*(exp(c2*(-t-dt))+1));

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = 0.0;
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset))+exp(2*c2*(-t+toff+tOffOffset))))/2;

costVec = act - act_objective;

end

function costVec = stimParamSqCostFunc(unknownsVec,act_objective,t,ton,toff)

% t = linspace(-1,1,101)';
% dt = unknownsVec(1);
% c1 = unknownsVec(2);
% c2 = unknownsVec(3);
% 
% act = 1./((exp(c1*(t-dt))+1).*(exp(c2*(-t-dt))+1));

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = unknownsVec(7);
tOffOffset2 = unknownsVec(8);
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset2))+exp(2*c2*(-t+toff+tOffOffset2))))/2;

costVec = sum((act - act_objective).^2);

end


function [c,cEq] = nonlinearConstraints(unknownsVec,t,ton,toff)

c1 = unknownsVec(1);
c2 = unknownsVec(2);
c3 = unknownsVec(3);
c4 = unknownsVec(4);
tOnOffset = unknownsVec(5);
tOnOffset2 = unknownsVec(6);
tOffOffset = unknownsVec(7);
tOffOffset2 = unknownsVec(8);
act = (tanh(c1*(t-ton-tOnOffset)) - tanh(c2*(t-toff-tOffOffset)) + ...
    c3./(exp(2*c1*(t-ton-tOnOffset2))+exp(2*c1*(-t+ton+tOnOffset2))) + ...
    c4./(exp(2*c2*(t-toff-tOffOffset2))+exp(2*c2*(-t+toff+tOffOffset2))))/2;

c = [-act;act-1]; % c <= 0, and we want 0 <= act <= 1;
cEq = []; % No non-linear equality constraints
end


