function [pR,pL,flags] = getWristMarkerLocs()
% Marker Plate Locations:
markerPlateR = [-0.0854172 -0.0854172 -0.0854172];
markerPlateL = [-0.100415666666667 -0.100415666666667 -0.100415666666667];

% Marker locations in marker plates:
markerInPlateR = [0.0496366 -0.152137 0.1025];
markerInPlateL = [0.065291 -0.150808 0.0855174];

% Calculate marker locations in radius:
markerInRadiusR = markerPlateR + markerInPlateR;
markerInRadiusL = markerPlateL + markerInPlateL;

% Calculate average wrist marker location:
MkrDataFile = 'Trial04_cycle_12.trc'; % Must be TRC file
[mkrData, mkrNames, ~, ~] = readTRCFile(MkrDataFile);

wristMkrNames = {'R.Wrist' 'L.Wrist'};
wristMkrIndex = zeros(1,numel(wristMkrNames));
for i = 1:numel(wristMkrNames)
    wristMkrIndex(i) = find(ismember(mkrNames, wristMkrNames{i}));
end
avgMarkerPosR = mean(mkrData(:,3*(wristMkrIndex(1)-1)+1:3*wristMkrIndex(1)),1);
avgMarkerPosL = mean(mkrData(:,3*(wristMkrIndex(2)-1)+1:3*wristMkrIndex(2)),1);

handMkrNames = {'R.Handle','L.Handle'};
handleMkrIndex = zeros(1,numel(handMkrNames));
for i = 1:numel(handMkrNames)
    handleMkrIndex(i) = find(ismember(mkrNames, handMkrNames{i}));
end
avgHandlePosR = mean(mkrData(:,3*(handleMkrIndex(1)-1)+1:3*handleMkrIndex(1)),1);
avgHandlePosL = mean(mkrData(:,3*(handleMkrIndex(2)-1)+1:3*handleMkrIndex(2)),1);

v1 = avgHandlePosL - avgHandlePosR;
v2R = avgMarkerPosR - avgHandlePosR;
v2L = avgMarkerPosL - avgHandlePosL;
v3R = cross(v1,v2R);
v3L = cross(v1,v2L);
v4R = cross(v1,v3R);
v4L = cross(v1,v3L);

% P1 = avgHandlePosR, P2 = avgMarkerPos
c2R = (avgHandlePosR(2) - avgMarkerPosR(2) + (avgMarkerPosR(1)-avgHandlePosR(1))*v1(2)/v1(1))/(v4R(2) - v4R(1)*v1(2)/v1(1));
c2L = (avgHandlePosR(2) - avgMarkerPosL(2) + (avgMarkerPosL(1)-avgHandlePosR(1))*v1(2)/v1(1))/(v4L(2) - v4L(1)*v1(2)/v1(1));

pR = avgMarkerPosR + c2R*v4R;
pL = avgMarkerPosL + c2L*v4L;

% Check results for consistency
c1R = (avgMarkerPosR(1) + c2R*v4R(1) - avgHandlePosR(1))/v1(1);
c1L = (avgMarkerPosL(1) + c2L*v4L(1) - avgHandlePosR(1))/v1(1);
pRCheck = avgHandlePosR + c1R*v1;
pLCheck = avgHandlePosR + c1L*v1;

checkR = sum(pR ~= pRCheck);
checkL = sum(pL ~= pLCheck);

flags = [checkR,checkL];

end

function [MarkerData, MarkerNames, SampleRate, Time] = readTRCFile(inFile)

% Read 5 header lines from input .trc file
fid = fopen(inFile);
header1 = fgetl(fid);
header2 = fgetl(fid);
header3 = fgetl(fid);
header4 = fgetl(fid);
header5 = fgetl(fid);

% Read marker data as one long column
data = fscanf(fid, '%f');
fclose(fid);

% Reshape marker data based on number of rows and
% columns of marker data specified in the input data
% file, with two extra columns for frame and time
info = sscanf(header3,'%f %f %d %d');
nrows = info(3,1);
ncols = info(4,1)*3;
data = reshape(data, ncols+2, nrows)';

SampleRate = info(1,1);

MarkerNames = strrep(header4,'Patient 4:','');
MarkerNames = textscan(MarkerNames,'%s');
MarkerNames = MarkerNames{1};
MarkerNames(1:2) = [];

Time = data(:,2);
MarkerData = data(:,3:end);
end